@@ -171,7 +171,7 @@ class ProxyManager:
                             print("success rate:", self.success_rate())
 
         task = Task(task_fn)
-        self._executor_queue.put(task)
+        self._executor_queue.put(task, True)
 
         if wait:
             return task.get()
@@ -179,23 +179,27 @@ class ProxyManager:
     async def start(self):
         self._pool.start()
 
-        def fn():
+        def fn(index: int):
             while self._executor_running:
                 try:
-                    task = self._executor_queue.get(False, timeout=1)
+                    task = self._executor_queue.get(False)
                     task()
 
                     self._executor_queue.task_done()
+                    continue
                 except Empty:
-                    time.sleep(1)
+                    time.sleep(3)
                     continue
 
-        self._executor_running = True
-        for _ in range(self._executor_workers):
-            self._executor.submit(fn)
+            if self._verbose:
+                print("Worker {} is stopped working".format(index))
 
-    async def stop(self, gracefully=True):
-        if gracefully:
+        self._executor_running = True
+        for i in range(self._executor_workers):
+            self._executor.submit(fn, i)
+
+    async def stop(self, forced_shutdown=False):
+        if not forced_shutdown:
             self._executor_queue.join()
 
         self._executor_running = False
@@ -211,14 +215,14 @@ class ProxyManager:
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         loop = asyncio.get_event_loop()
-        return loop.run_until_complete(self.stop(exc_type is None))
+        return loop.run_until_complete(self.stop(exc_type is not None))
 
     async def __aenter__(self):
         await self.start()
         return self
 
     async def __aexit__(self, exc_type, exc_val, exc_tb):
-        return await self.stop(exc_type is None)
+        return await self.stop(exc_type is not None)
 
 
 class SessionProxy(Session):
